// ======================================================================
//
// gl_3dfx.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef GL_3DFX_H
#define GL_3DFX_H

// ======================================================================

#include <ddraw.h>
#include <glide.h>

#include "Camera.h"
#include "Gl_abstract.h"
#include "Gl.def"

class PaletteData;
class PaletteData_3dfx;
class MaterialData_3dfx;
class MemoryBlockManager;

// ======================================================================

class GL_3dfx : public GL_abstract
{
private:

	// disable copy constructor and assignment operator
	GL_3dfx(const GL_3dfx &);
	GL_3dfx &operator =(const GL_3dfx &);

private:

	enum
	{
		R_PLACE_SHIFT = 11,
		G_PLACE_SHIFT = 5,
		B_PLACE_SHIFT = 0,

		R_SIZE_SHIFT  = 3,
		G_SIZE_SHIFT  = 2,
		B_SIZE_SHIFT  = 3
	};

private:

	static GrHwConfiguration               hwConfig;
	static char                            version[80];
	static char                            informationName[128];

	static FxU32                           memBase;
	static FxU32                           memTop;
	static FxU32                           memTotal;

	static PrimitiveProcessing             primitiveProcessing;

	static LPDIRECTDRAW                    ddraw;

	static bool                            isVoodoo;
	static bool                            active;
	static bool                            aaLine;
	static bool                            materialTranslucent;

	static int                             textureDownloads;
	static int                             paletteDownloads;

private:

	static void myDrawLine(const GrVertex *v0, const GrVertex *v1);

	static void __cdecl error(const char *str, FxBool fatal);

	// submit transformed/clipped primitives to the rasterizer
	static void passPointList(const TVertex *vertexList, int numberOfVertices);
	static void passLineList(const TVertex *vertexList, int numberOfVertices);
	static void passLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void passTriangleFan(const TVertex *vertexList, int numberOfVertices);

	static void testPointList(const TVertex *vertexList, int numberOfVertices);
	static void testLineList(const TVertex *vertexList, int numberOfVertices);
	static void testLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void testTriangleFan(const TVertex *vertexList, int numberOfVertices);

	static void queuePointList(const TVertex *vertexList, int numberOfVertices);
	static void queueLineList(const TVertex *vertexList, int numberOfVertices);
	static void queueLineStrip(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleList(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	static void queueTriangleFan(const TVertex *vertexList, int numberOfVertices);

public:

	static void countDownloadedTexture(void);
	static void countDownloadedPalette(void);

public:

	GL_3dfx(void);
	virtual ~GL_3dfx(void);
	
	// version information
	virtual void  getInformation(Information *information);

	// see if the is usable
	virtual bool  query(int minor);

	// initialize rasterizer
	virtual bool init(const GL_input *input, GL_output *output);

	// application activation
	virtual void setActive(bool newActive);

	// debugging functions
	virtual void debugDump();

	// access functions
	virtual bool lock(void);
	virtual void unlock(bool display);
	virtual bool gdiVisible(void);

	// set processing of primitives
	virtual void setAlphaBlendEnabled(bool enabled);
	virtual void setAlphaBlendFunctions(AlphaBlendFunction source, AlphaBlendFunction destination);
	virtual void setPrimitiveProcessing(PrimitiveProcessing primitiveProcessing);

	// palette data functions
	virtual PaletteData  *paletteCreateData(const Palette *pal);
	virtual void          paletteDestroyData(PaletteData *palData);

	// material data functions
	virtual MaterialData *materialCreateData(const StaticMaterial *material);
	virtual void          materialDestroyData(MaterialData *mat);

	// material functions
	virtual void setMaterial(const StaticMaterial *material);

	// 3d transformation setup
	virtual void beginRender(const Camera &camera);
	virtual void endRender(void);

	// 3d render setup
	virtual void setLightingEnabled(bool lightingEnabled);
	virtual void setLightList(Light **lightList, int lightCount);
	virtual void setObjectToWorld(const Xform &objectToWorld);
	virtual bool setVertexList(const Vector *vertex, int vertexCount, const Vector *normal, int normalCount);

	// object-space primitives
	virtual void drawPointList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const UVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const UVertex *vertexList, int numberOfVertices);

	// screen-space primitives
	virtual void drawPointList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawLineStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleList(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleStrip(const TVertex *vertexList, int numberOfVertices);
	virtual void drawTriangleFan(const TVertex *vertexList, int numberOfVertices);

	// movie primtive
	virtual void  movie(const uv_instance *uv);

	// per-rasterizer special effects
	virtual bool specialEffect(int id, void *data);
};

// ======================================================================

inline void GL_3dfx::countDownloadedTexture(void)
{
	++textureDownloads;
}

// ----------------------------------------------------------------------

inline void GL_3dfx::countDownloadedPalette(void)
{
	++paletteDownloads;
}

// ======================================================================

#endif
